(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common')) :
    typeof define === 'function' && define.amd ? define('ngx-keyboard-shortcuts', ['exports', '@angular/core', '@angular/common'], factory) :
    (global = global || self, factory(global['ngx-keyboard-shortcuts'] = {}, global.ng.core, global.ng.common));
}(this, function (exports, core, common) { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */

    function __decorate(decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
        else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }

    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(metadataKey, metadataValue);
    }

    function __values(o) {
        var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
        if (m) return m.call(o);
        return {
            next: function () {
                if (o && i >= o.length) o = void 0;
                return { value: o && o[i++], done: !o };
            }
        };
    }

    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m) return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
        }
        catch (error) { e = { error: error }; }
        finally {
            try {
                if (r && !r.done && (m = i["return"])) m.call(i);
            }
            finally { if (e) throw e.error; }
        }
        return ar;
    }

    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }

    var KeyboardShortcutListener = /** @class */ (function () {
        function KeyboardShortcutListener(listenerConstructorObject, blackListedKeyboardShortcutChecker) {
            // assign properties based on constructor and defaults
            var defaultListenerOptions = {
                ignoreEventsFromInputElement: false,
                passToLowerPriorities: true,
                priority: 0
            };
            Object.assign(this, defaultListenerOptions, listenerConstructorObject);
            // assign mapped keyboard shortcut combination
            this.mappedKeyboardShortcutCombination = mapKeyboardShortcutCombination(this.keyBinding);
            // assign display shortcut combination
            var copyOfKeyBinding = __spread(this.keyBinding);
            copyOfKeyBinding.sort(function (a, b) {
                if (a.length > 1 && b.length === 1) {
                    return -1;
                }
                else if (a.length === 1 && b.length > 1) {
                    return 1;
                }
                else if (a > b) {
                    return 1;
                }
                else if (a < b) {
                    return -1;
                }
                else {
                    return 0;
                }
            });
            this.displayShortcutCombination = copyOfKeyBinding
                .map(function (kb) { return kb.replace(/^\w/, function (c) { return c.toUpperCase(); }); })
                .join(' + ');
            // check vs blacklist
            if (blackListedKeyboardShortcutChecker) {
                blackListedKeyboardShortcutChecker.check(this);
            }
        }
        return KeyboardShortcutListener;
    }());
    /**
     * @enum this allows you to map special keys into your Keyboard Combination
     */

    (function (KeyboardKeys) {
        KeyboardKeys["Ctrl"] = "ctrl";
        KeyboardKeys["Alt"] = "alt";
        KeyboardKeys["Shift"] = "shift";
        KeyboardKeys["Escape"] = "escape";
    })(exports.KeyboardKeys || (exports.KeyboardKeys = {}));
    /**
     * @param bindings takes in the shortcut from KeyboardShortcutCombination which is an array of strings
     * @return {string}
     */
    function mapKeyboardShortcutCombination(bindings) {
        return JSON.stringify(bindings.map(function (key) { return key.toLowerCase(); }).sort());
    }
    // Map to normalized keys across different browser implementations.
    // https://github.com/angular/angular/blob/5.0.5/packages/platform-browser/src/browser/browser_adapter.ts#L25-L42
    // tslint:disable:object-literal-sort-keys
    /**
     * @constant
     * Map to convert some key or keyIdentifier values to what will be returned by getEventKey
     * This is also used to help browser compatibility and to match the W3C standard
     */
    var KEY_MAP = {
        '\b': 'backspace',
        '\t': 'tab',
        '\x1B': 'escape',
        '\x7F': 'delete',
        Del: 'delete',
        Esc: 'escape',
        Left: 'arrowleft',
        Right: 'arrowright',
        Up: 'arrowup',
        Down: 'arrowdown',
        Menu: 'contextMenu',
        Scroll: 'scrolllock',
        Win: 'os',
        ' ': 'space',
        '.': 'dot'
    };
    // tslint:enable:object-literal-sort-keys

    var blacklistedKeyCombinations = [
        [exports.KeyboardKeys.Ctrl, 'd'],
        [exports.KeyboardKeys.Ctrl, 'f'],
        [exports.KeyboardKeys.Ctrl, 'g'],
        [exports.KeyboardKeys.Ctrl, exports.KeyboardKeys.Shift, 'g'],
        [exports.KeyboardKeys.Ctrl, 'h'],
        [exports.KeyboardKeys.Ctrl, 'j'],
        [exports.KeyboardKeys.Ctrl, 'o'],
        [exports.KeyboardKeys.Ctrl, 's'],
        [exports.KeyboardKeys.Ctrl, 'p'],
        [exports.KeyboardKeys.Ctrl, 'u'],
        [exports.KeyboardKeys.Ctrl, 'n'],
        [exports.KeyboardKeys.Ctrl, 't'],
        [exports.KeyboardKeys.Ctrl, exports.KeyboardKeys.Shift, 't'],
        [exports.KeyboardKeys.Ctrl, 'c'],
        [exports.KeyboardKeys.Ctrl, 'x'],
        [exports.KeyboardKeys.Ctrl, 'v'],
        [exports.KeyboardKeys.Alt, 'f'],
        [exports.KeyboardKeys.Ctrl, '0'],
        [exports.KeyboardKeys.Ctrl, '1'],
        [exports.KeyboardKeys.Ctrl, '2'],
        [exports.KeyboardKeys.Ctrl, '3'],
        [exports.KeyboardKeys.Ctrl, '4'],
        [exports.KeyboardKeys.Ctrl, '5'],
        [exports.KeyboardKeys.Ctrl, '6'],
        [exports.KeyboardKeys.Ctrl, '7'],
        [exports.KeyboardKeys.Ctrl, '8'],
        [exports.KeyboardKeys.Ctrl, '9'],
        [exports.KeyboardKeys.Shift, '1'],
        [exports.KeyboardKeys.Shift, '2'],
        [exports.KeyboardKeys.Shift, '3'],
        [exports.KeyboardKeys.Shift, '4'],
        [exports.KeyboardKeys.Shift, '5'],
        [exports.KeyboardKeys.Shift, '6'],
        [exports.KeyboardKeys.Shift, '7'],
        [exports.KeyboardKeys.Shift, '8'],
        [exports.KeyboardKeys.Shift, '9'],
        [exports.KeyboardKeys.Shift, '0'],
        [exports.KeyboardKeys.Escape],
        [exports.KeyboardKeys.Ctrl, 'tab'],
        [exports.KeyboardKeys.Ctrl, exports.KeyboardKeys.Shift, 'tab'],
        [exports.KeyboardKeys.Ctrl, 'w'],
        [exports.KeyboardKeys.Alt, '4'],
        [exports.KeyboardKeys.Alt, 'arrowleft'],
        [exports.KeyboardKeys.Alt, 'backspace'],
        [exports.KeyboardKeys.Ctrl, exports.KeyboardKeys.Shift, 'delete'] // open the clear browsing history window
    ];
    var BlackListedKeyboardShortcutChecker = /** @class */ (function () {
        function BlackListedKeyboardShortcutChecker() {
            this.blackListedKeyBindings = blacklistedKeyCombinations;
            this.mappedBlackListedKeyBindings = this.blackListedKeyBindings.map(function (blkb) { return mapKeyboardShortcutCombination(blkb); });
        }
        BlackListedKeyboardShortcutChecker.prototype.check = function (listener) {
            var e_1, _a;
            try {
                // this is where the test was deleted
                for (var _b = __values(this.mappedBlackListedKeyBindings), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var blkb = _c.value;
                    if (blkb === listener.mappedKeyboardShortcutCombination) {
                        var warningMessage = "Keyboard Shortcut [" + listener.displayShortcutCombination + "] is blacklisted as a common keyboard shortcut.  Consider changing your keybinding for this shortcut.";
                        console.warn(warningMessage);
                        return;
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
        };
        return BlackListedKeyboardShortcutChecker;
    }());

    var KeyboardShortcutService = /** @class */ (function () {
        function KeyboardShortcutService(zone) {
            var _this = this;
            this._listeners = [];
            this.zone = zone;
            // Since we're going to create a root event-handler for the keydown event, we're
            // gonna do this outside of the NgZone. This way, we're not constantly triggering
            // change-detection for every key event - we'll only re-enter the Angular Zone
            // when we have an event that is actually being consumed by one of our components.
            this.zone.runOutsideAngular(function () {
                window.addEventListener('keydown', _this.handleKeyboardEvent.bind(_this));
            });
            // we only use this if isDevMode
            if (core.isDevMode()) {
                this.blackListedKeyboardShortcutChecker = new BlackListedKeyboardShortcutChecker();
            }
        }
        Object.defineProperty(KeyboardShortcutService.prototype, "listeners_read_only", {
            /**
             * @readonly
             * @description A list of all available shortcut listeners
             * @property    {IKeyboardShortcutListener[]}
             */
            get: function () {
                return JSON.parse(JSON.stringify(this._listeners));
            },
            enumerable: true,
            configurable: true
        });
        /**
         * @description                     Creates a listenerHandle using imported interface
         * @param listenerConstructorObject Interface that contains a KeyboardShortcutHandler
         * @returns                         an IListenerHandle
         */
        KeyboardShortcutService.prototype.listen = function (listenerConstructorObject) {
            var listenerHandle = this.addListener(listenerConstructorObject);
            return listenerHandle;
        };
        KeyboardShortcutService.prototype.addListener = function (listenerConstructorObject) {
            var _this = this;
            var listener = new KeyboardShortcutListener(listenerConstructorObject, this.blackListedKeyboardShortcutChecker);
            var listenerHandle = {
                remove: function () {
                    _this.removeListener(listener);
                }
            };
            this._listeners.push(listener);
            this._listeners.sort(function (a, b) {
                if (a.priority < b.priority) {
                    return 1;
                }
                else if (a.priority > b.priority) {
                    return -1;
                }
                else {
                    return 0;
                }
            });
            return listenerHandle;
        };
        // get the keyCombination from the given event
        KeyboardShortcutService.prototype.getKeyFromEvent = function (event) {
            var keyIdentifier = 'keyIdentifier';
            var key = event.key || event[keyIdentifier] || 'Unidentified';
            if (key.startsWith('U+')) {
                key = String.fromCharCode(parseInt(key.slice(2), 16));
            }
            var parts = [(KEY_MAP[key] || key)];
            if (event.altKey) {
                parts.push(exports.KeyboardKeys.Alt);
            }
            if (event.ctrlKey) {
                parts.push(exports.KeyboardKeys.Ctrl);
            }
            if (event.shiftKey) {
                parts.push(exports.KeyboardKeys.Shift);
            }
            return mapKeyboardShortcutCombination(parts);
        };
        // handle the keyboard events for the root handler (and delegate to the listeners).
        KeyboardShortcutService.prototype.handleKeyboardEvent = function (event) {
            var e_1, _a;
            var keyCombination = this.getKeyFromEvent(event);
            var isInputEvent = this.isEventFromInput(event);
            var handler;
            try {
                // Iterate over the listeners in DESCENDING priority order.
                for (var _b = __values(this._listeners), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var listener = _c.value;
                    if (listener.mappedKeyboardShortcutCombination === keyCombination) {
                        handler = listener.handler;
                        // Execute handler if this is NOT an input event that we need to ignore.
                        if (!isInputEvent || !listener.ignoreEventsFromInputElement) {
                            // Right now, we're executing outside of the NgZone. As such, we
                            // have to re-enter the NgZone so that we can hook back into change-
                            // detection. Plus, this will also catch errors and propagate them
                            // through application properly.
                            this.zone.runGuarded(function () {
                                return handler(event);
                            });
                            // If the handler returned an explicit False, we're going to treat
                            // this listener as Terminal, regardless of the original settings.
                            if (!listener.passToLowerPriorities) {
                                return;
                            }
                            else {
                                continue;
                            }
                        }
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
        };
        /**
         * @description Used for testing, Sends a keyboard event
         * @param event a keyboard event
         */
        KeyboardShortcutService.prototype.sendKeyboardEventToHandler = function (event) {
            this.handleKeyboardEvent(event);
        };
        // determine if the given event originated from a form input element.
        KeyboardShortcutService.prototype.isEventFromInput = function (event) {
            if (event.target instanceof Node) {
                switch (event.target.nodeName) {
                    case 'INPUT':
                    case 'SELECT':
                    case 'TEXTAREA':
                        return true;
                    default:
                        return false;
                }
            }
            return false;
        };
        // remove the given listener from the internal collection.
        KeyboardShortcutService.prototype.removeListener = function (listenerToRemove) {
            this._listeners = this._listeners.filter(function (listener) {
                return listener !== listenerToRemove;
            });
        };
        KeyboardShortcutService = __decorate([
            core.Injectable(),
            __metadata("design:paramtypes", [core.NgZone])
        ], KeyboardShortcutService);
        return KeyboardShortcutService;
    }());

    var KeyboardShortcutDirective = /** @class */ (function () {
        function KeyboardShortcutDirective(elRef, keyboardShortcutService) {
            this.elRef = elRef;
            this.keyboardShortcutService = keyboardShortcutService;
            this.fireClickEventOnKeyboardShortcut = true;
            this.onKeyboardShortcut = new core.EventEmitter();
        }
        KeyboardShortcutDirective.prototype.ngOnInit = function () {
            if (!this.keyboardShortcut) {
                return;
            }
            var listenerConstructor = {};
            Object.assign(listenerConstructor, { handler: this.keyboardShortcutHandler.bind(this) }, this.keyboardShortcut);
            this.listener = this.keyboardShortcutService.listen(listenerConstructor);
        };
        KeyboardShortcutDirective.prototype.keyboardShortcutHandler = function (event) {
            if (this.fireClickEventOnKeyboardShortcut) {
                if (this.elRef &&
                    this.elRef.nativeElement &&
                    this.elRef.nativeElement.click) {
                    this.elRef.nativeElement.click();
                }
            }
            this.onKeyboardShortcut.emit(event);
        };
        KeyboardShortcutDirective.prototype.ngOnDestroy = function () {
            if (this.listener) {
                this.listener.remove();
            }
        };
        __decorate([
            core.Input(),
            __metadata("design:type", Object)
        ], KeyboardShortcutDirective.prototype, "keyboardShortcut", void 0);
        __decorate([
            core.Input(),
            __metadata("design:type", Object)
        ], KeyboardShortcutDirective.prototype, "fireClickEventOnKeyboardShortcut", void 0);
        __decorate([
            core.Output(),
            __metadata("design:type", Object)
        ], KeyboardShortcutDirective.prototype, "onKeyboardShortcut", void 0);
        KeyboardShortcutDirective = __decorate([
            core.Directive({
                selector: '[keyboardShortcut]'
            }),
            __metadata("design:paramtypes", [core.ElementRef,
                KeyboardShortcutService])
        ], KeyboardShortcutDirective);
        return KeyboardShortcutDirective;
    }());

    var NgxKeyboardShortcutModule = /** @class */ (function () {
        function NgxKeyboardShortcutModule() {
        }
        NgxKeyboardShortcutModule_1 = NgxKeyboardShortcutModule;
        NgxKeyboardShortcutModule.forRoot = function () {
            return {
                ngModule: NgxKeyboardShortcutModule_1,
                providers: [
                    KeyboardShortcutService
                ]
            };
        };
        var NgxKeyboardShortcutModule_1;
        NgxKeyboardShortcutModule = NgxKeyboardShortcutModule_1 = __decorate([
            core.NgModule({
                declarations: [KeyboardShortcutDirective],
                exports: [KeyboardShortcutDirective],
                imports: [common.CommonModule]
            })
        ], NgxKeyboardShortcutModule);
        return NgxKeyboardShortcutModule;
    }());

    exports.KEY_MAP = KEY_MAP;
    exports.KeyboardShortcutDirective = KeyboardShortcutDirective;
    exports.KeyboardShortcutListener = KeyboardShortcutListener;
    exports.KeyboardShortcutService = KeyboardShortcutService;
    exports.NgxKeyboardShortcutModule = NgxKeyboardShortcutModule;
    exports.blacklistedKeyCombinations = blacklistedKeyCombinations;
    exports.mapKeyboardShortcutCombination = mapKeyboardShortcutCombination;
    exports.ɵa = BlackListedKeyboardShortcutChecker;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=ngx-keyboard-shortcuts.umd.js.map
